package main

import (
	"log"
	"sync"
)

// WorkerContext 上下文
type WorkerContext struct {
	wg *sync.WaitGroup
}

// Done 必须在Handler里结束时处理. 默认defer cxt.Done()
func (cxt *WorkerContext) Done() {
	cxt.wg.Done()
}

// Worker 主进程
type Worker struct {
	wg  *sync.WaitGroup
	cxt *WorkerContext
}

var worker = func() *Worker {
	w := &Worker{}
	w.cxt = &WorkerContext{}
	w.wg = &sync.WaitGroup{}
	return w
}()

// Handler 处理方法
func (w *Worker) Handler(handleFunc func(cxt *WorkerContext)) {
	w.wg.Add(1)
	go handleFunc(w.cxt)
}

// Run 运行
func (w *Worker) Run() {
	log.Println("worker running")
	w.wg.Wait()
	log.Println("worker stop")
}
